import { OpenSheetMusicDisplay, Instrument, Voice } from "opensheetmusicdisplay";
import { InstrumentPlayer } from "./players/InstrumentPlayer";
export declare enum PlaybackState {
    INIT = "INIT",
    PLAYING = "PLAYING",
    STOPPED = "STOPPED",
    PAUSED = "PAUSED"
}
export declare enum PlaybackEvent {
    STATE_CHANGE = "state-change",
    ITERATION = "iteration"
}
interface PlaybackSettings {
    bpm: number;
    masterVolume: number;
}
export default class PlaybackEngine {
    private ac;
    private defaultBpm;
    private cursor;
    private sheet;
    private denominator;
    private scheduler;
    private instrumentPlayer;
    private events;
    private iterationSteps;
    private currentIterationStep;
    private timeoutHandles;
    playbackSettings: PlaybackSettings;
    state: PlaybackState;
    availableInstruments: [number, string][];
    scoreInstruments: Instrument[];
    ready: boolean;
    constructor(context?: AudioContext, instrumentPlayer?: InstrumentPlayer);
    get wholeNoteLength(): number;
    getPlaybackInstrument(voiceId: number): [number, string];
    setInstrument(voice: Voice, midiInstrumentId: number): Promise<void>;
    loadScore(osmd: OpenSheetMusicDisplay): Promise<void>;
    private initInstruments;
    private loadInstruments;
    play(): Promise<void>;
    stop(): Promise<void>;
    pause(): void;
    jumpToStep(step: any): void;
    setBpm(bpm: number): void;
    on(event: PlaybackEvent, cb: (...args: any[]) => void): void;
    private countAndSetIterationSteps;
    private notePlaybackCallback;
    private setState;
    private stopPlayers;
    private clearTimeouts;
    private iterationCallback;
}
export {};
